@@ -1,12 +1,12 @@
 ;;; jit-spell.el --- Just-in-time spell checking      -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2023  Free Software Foundation, Inc.
+;; Copyright (C) 2023-2024  Free Software Foundation, Inc.
 
 ;; Author: Augusto Stoffel <arstoffel@gmail.com>
 ;; Keywords: tools, wp
 ;; URL: https://github.com/astoff/jit-spell
 ;; Package-Requires: ((emacs "27.1") (compat "29.1"))
-;; Version: 0.3
+;; Version: 0.4
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -187,9 +187,10 @@ character offset from START, and a list of corrections."
     (catch 'jit-spell
       (while (/= pos limit)
         (setq pos (funcall searchfn pos 'category nil limit))
-        (when-let ((ov (jit-spell--overlay-at pos)))
-          (unless (cl-plusp (cl-decf i))
-            (throw 'jit-spell ov)))))))
+        (unless (invisible-p pos)
+          (when-let ((ov (jit-spell--overlay-at pos)))
+            (unless (cl-plusp (cl-decf i))
+              (throw 'jit-spell ov))))))))
 
 (defun jit-spell--remove-overlays (start end &optional gaps)
   "Remove all `jit-spell' overlays between START and END, skipping GAPS.
@@ -492,7 +493,11 @@ With a numeric ARG, move backwards that many misspellings."
   (interactive "p")
   (if (minibufferp)
       (throw 'jit-spell--next arg)
-    (named-let recur ((count (- arg)) (pos (point)))
+    (named-let recur ((count (- arg))
+                      (pos (if (cl-minusp arg)
+                               (point)
+                             ;; Catch misspelling at point (bug #6)
+                             (min (1+ (point)) (point-max)))))
       (let* ((ov (or (jit-spell--search-overlay pos count)
                      (user-error "No more misspellings")))
              (start (overlay-start ov))
@@ -529,6 +534,56 @@ With a numeric ARG, move backwards that many misspellings."
            (jit-spell--accept-word (or (match-string 1 corr) word) 'query))
           (corr (jit-spell--apply-correction ov corr)))))))
 
+(defun jit-spell-remove-word (word)
+  "Remove the WORD from personal dictionary or local word lists.
+Interactively offer the current word or sub-words.
+
+To remove words from the personal dictionary `ispell-personal-dictionary'
+must be set.  The location of personal dictionary depends on the spell checking
+program and there is no reliable way of obtaining it."
+  (interactive (list (let ((cand (current-word)))
+                       (completing-read
+                        "Remove word: "
+                        (append `(,cand) (string-split cand (rx (not alpha))))
+                        nil nil nil nil cand))))
+  (let ((places nil))
+    ;; Check among the local words.
+    (save-excursion
+      (goto-char (point-min))
+      (when (re-search-forward (rx bol (syntax comment-start) (*? nonl)
+                                   (literal ispell-words-keyword) (*? nonl)
+                                   (group-n 1 (and " " (literal word))))
+                               nil t)
+        (push "the buffer's word list" places)
+        (delete-region (match-beginning 1) (match-end 1))))
+    ;; Remove it from the local word list.
+    (when (member word jit-spell--local-words)
+      (cl-callf2 delete word jit-spell--local-words)
+      (unless places (push "the session's word list" places)))
+    ;; Remove it from the personal dictionary
+    (let ((case-fold-search nil))
+      (when (and ispell-personal-dictionary
+                 (file-exists-p ispell-personal-dictionary))
+        (with-temp-buffer
+          (insert-file-contents ispell-personal-dictionary nil nil nil t)
+          (when (re-search-forward (rx bol (literal word)) nil t)
+            (delete-line)
+            (let ((inhibit-message t))
+              (write-file ispell-personal-dictionary))
+            (push "the personal dictionary" places)
+            (let* ((proc (jit-spell--get-process))
+                   (pending (process-get proc 'jit-spell--requests)))
+              (kill-process proc)
+              (mapc (pcase-lambda (`(,buf _ _ _))
+                      (with-current-buffer buf (jit-lock-refontify)))
+                    pending))))))
+    (if (not places)
+        (message
+         "\"%s\" not found in the personal dictionary or local word lists."
+         word)
+      (message "\"%s\" removed from %s." word (string-join places " and "))
+      (jit-lock-refontify))))
+
 (defalias 'jit-spell-change-dictionary 'ispell-change-dictionary) ;For discoverability
 
 ;;; Minor mode definition
@@ -582,7 +637,7 @@ It can also be bound to a mouse click to pop up the menu."
      ((derived-mode-p 'prog-mode)
       (add-function :filter-return (local 'jit-spell--filter-region)
                     (jit-spell--refine-by-face jit-spell-prog-mode-faces t)))
-     ((derived-mode-p 'tex-mode 'context-mode)
+     ((derived-mode-p 'tex-mode 'TeX-mode 'context-mode)
       (add-function :filter-return (local 'jit-spell--filter-region)
                     (jit-spell--refine-by-face jit-spell-tex-ignored-faces))))
     ;; Generic ignore predicate
